#include "../Includes/qstdinc.h"
#include "../Includes/qtime.h"

uint64_t QTimeFromTime(time_t tSrc)
{
	struct tm pti;
#ifndef _WIN32
	localtime_r(&tSrc, &pti);
#else
	localtime_s(&pti, &tSrc);
#endif
	return  QTimeCreate(pti.tm_year+1900, pti.tm_mon+1, pti.tm_mday, pti.tm_hour, pti.tm_min, pti.tm_sec);
}

uint64_t QTimeFromTimespec(timespec& tSrc)
{
	struct tm pti;
#ifndef _WIN32
	localtime_r(&tSrc.tv_sec, &pti);
#else
	localtime_s(&pti, &tSrc.tv_sec);
#endif
	return QTimeCreate(pti.tm_year+1900, pti.tm_mon+1, pti.tm_mday, pti.tm_hour, pti.tm_min, pti.tm_sec, tSrc.tv_nsec/10000000);
}

time_t QTimeToTime(uint64_t tSrc)
{
	struct tm tDst;

	memset(&tDst, 0, sizeof(tDst));
	tDst.tm_isdst = -1;

	tDst.tm_year = QTimeYear(tSrc); if (tDst.tm_year >= 1900) tDst.tm_year -= 1900; else tDst.tm_year = 0;
	tDst.tm_mon = QTimeMonth(tSrc) - 1;
	tDst.tm_mday = QTimeDay(tSrc);

	tDst.tm_hour = QTimeHour(tSrc);
	tDst.tm_min = QTimeMinute(tSrc);
	tDst.tm_sec = QTimeSecond(tSrc);
	return mktime(&tDst);
}

timespec QTimeToTimespec(uint64_t tSrc)
{
	timespec tDst;
	tDst.tv_sec = QTimeToTime(tSrc);
	tDst.tv_nsec = QTimeSecondFraq(tSrc) * 10000000;
	return tDst;
}

uint64_t QTimeNow()
{
#ifndef _WIN32
	timespec tSrc;
	struct tm pti;

	if (clock_gettime(CLOCK_REALTIME, &tSrc) != 0)
	{
	}

	// Getting current timeinfo
	localtime_r(&tSrc.tv_sec, &pti);

	return QTimeCreate(pti.tm_year+1900, pti.tm_mon+1, pti.tm_mday, pti.tm_hour, pti.tm_min, pti.tm_sec, tSrc.tv_nsec/10000000);
#else
	SYSTEMTIME st;
	GetLocalTime(&st);

	return QTimeCreate(st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds / 10);
#endif
}

uint64_t QTimeCreate(int year, int month, int day, int hour, int minute, int second, int sfraq)
{
	uint64_t mt;
	mt = year;
	mt <<= 8;
	mt += month;
	mt <<= 8;
	mt += day;
	mt <<= 8;
	mt += hour;
	mt <<= 8;
	mt += minute;
	mt <<= 8;
	mt += second;
	mt <<= 8;
	mt += sfraq;

	return mt;
}

int QTimeYear(uint64_t mt)
{
	return (int)(mt >> 48);
}

int QTimeMonth(uint64_t mt)
{
	return (int)((mt >> 40) & 0xFF);
}

int QTimeDay(uint64_t mt)
{
	return (int)((mt >> 32) & 0xFF);
}

int QTimeHour(uint64_t mt)
{
	return (int)((mt >> 24) & 0xFF);
}

int QTimeMinute(uint64_t mt)
{
	return (int)((mt >> 16) & 0xFF);
}

int QTimeSecond(uint64_t mt)
{
	return (int)((mt >> 8) & 0xFF);
}

int QTimeSecondFraq(uint64_t mt)
{
	return (int)(mt  & 0xFF);
}

int QTimeDiffSecs(uint64_t t1, uint64_t t2)
{
	return (int)QTimeToTime(t2) - (int)QTimeToTime(t1);
}

uint64_t QTimeAddSecs(uint64_t tSrc, int nSecs)
{
	timespec tSpec = QTimeToTimespec(tSrc);
	tSpec.tv_sec += nSecs;
	return QTimeFromTimespec(tSpec);
}

uint64_t QTimeDiffSantiSecs(uint64_t nFrom, uint64_t nWhat)
{
	timespec tSpec1 = QTimeToTimespec(nFrom);
	timespec tSpec2 = QTimeToTimespec(nWhat);
	uint64_t nRetVal = QDiffTimespec(&tSpec1, &tSpec2);

	/* 1/100 sec */
	nRetVal = nRetVal / (uint64_t)10000000LL;
	return nRetVal;
}

uint64_t QTimeAddSantiSecs(uint64_t nTo, uint64_t nFraq)
{
	timespec tTime = QTimeToTimespec(nTo);
	tTime.tv_nsec = tTime.tv_nsec + (long)(nFraq * (uint64_t)10000000LL);
	uint64_t nSecs = tTime.tv_nsec / (uint64_t)1000000000LL;
	tTime.tv_sec += nSecs;
	tTime.tv_nsec = tTime.tv_nsec % (uint64_t)1000000000LL;

	return QTimeFromTimespec(tTime);
}

void QTimeToStr(char* pDst, uint64_t nTime)
{
	sprintf(pDst, "%04d%02d%02d%02d%02d%02d%02d", 
			QTimeYear(nTime), QTimeMonth(nTime),
			QTimeDay(nTime), QTimeHour(nTime),
			QTimeMinute(nTime), QTimeSecond(nTime),
			QTimeSecondFraq(nTime));
}

void QTimeToStrR(char* pDst, uint64_t nTime)
{
	sprintf(pDst, "%04d-%02d-%02d %02d:%02d:%02d.%02d", 
			QTimeYear(nTime), QTimeMonth(nTime),
			QTimeDay(nTime), QTimeHour(nTime),
			QTimeMinute(nTime), QTimeSecond(nTime),
			QTimeSecondFraq(nTime));
}

uint64_t QTimeFromStr(const char* pStr)
{
	int nYear, nMonth, nDay, nHour, nMinute, nSecond, nFraq;
	nYear = nMonth = nDay = nHour = nMinute = nSecond = nFraq = 0;

	sscanf(pStr, "%04d%02d%02d%02d%02d%02d%02d", &nYear, &nMonth, &nDay, &nHour, &nMinute, &nSecond, &nFraq);

	return QTimeCreate(nYear, nMonth, nDay, nHour, nMinute, nSecond, nFraq);
}

uint64_t QTimeFromStrNet(const char* pStr)
{
	int nYear, nMonth, nDay, nHour, nMinute, nSecond, nFraq;
	nYear = nMonth = nDay = nHour = nMinute = nSecond = nFraq = 0;

	sscanf(pStr, "%04d-%02d-%02d-%02d-%02d-%02d", &nYear, &nMonth, &nDay, &nHour, &nMinute, &nSecond);

	return QTimeCreate(nYear, nMonth, nDay, nHour, nMinute, nSecond, 0);
}

void QTimeToStrNet(char* pDst, uint64_t nTime)
{
	sprintf(pDst, "%04d-%02d-%02d-%02d-%02d-%02d",
			QTimeYear(nTime), QTimeMonth(nTime),
			QTimeDay(nTime), QTimeHour(nTime),
			QTimeMinute(nTime), QTimeSecond(nTime));
}


uint64_t QDiffTimespec(timespec* pNext, timespec* pPrev)
{
        uint64_t nDiff = pNext->tv_sec - pPrev->tv_sec;
        nDiff = nDiff * (uint64_t)1000000000LL;
        nDiff += ((uint64_t)pNext->tv_nsec - (uint64_t)pPrev->tv_nsec);
        return nDiff;
}

void QQ2GetHTTPTime(std::string& sLine, int nShift)
{
	/* Date: Thu, 01 Aug 2013 19:42:25 GMT */
	char sBuffer[128];
	time_t rawtime;
    time ( &rawtime );
	int32_t xRawTime = (int32_t)rawtime;

    xRawTime = xRawTime + (int32_t)nShift;
    rawtime = (time_t)xRawTime;

    struct tm timeinfo;
#ifndef _WIN32
    gmtime_r(&rawtime, &timeinfo);
#else
    gmtime_s(&timeinfo, &rawtime);
#endif
	strftime(sBuffer, 127, "%a, %d %b %G %H:%M:%S GMT", &timeinfo);

	sLine = std::string(sBuffer);
}

